/*
 * Global.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

using System;
using Eagle._Attributes;
using Eagle._Components.Private;
using Eagle._Components.Public;
using Eagle._Containers.Public;
using Eagle._Interfaces.Public;

namespace Eagle._Commands
{
    [ObjectId("1cd4e351-10d3-4e53-bcfa-6d7e09e41184")]
    [CommandFlags(CommandFlags.Safe | CommandFlags.Standard)]
    [ObjectGroup("variable")]
    internal sealed class Global : Core
    {
        public Global(
            ICommandData commandData
            )
            : base(commandData)
        {
            // do nothing.
        }

        #region IExecute Members
        public override ReturnCode Execute(
            Interpreter interpreter,
            IClientData clientData,
            ArgumentList arguments,
            ref Result result
            )
        {
            ReturnCode code = ReturnCode.Ok;

            if (interpreter != null)
            {
                if (arguments != null)
                {
                    if (arguments.Count >= 2)
                    {
                        ICallFrame currentFrame =
                            CallFrameOps.FollowLinks(interpreter.CurrentFrame);

                        if ((currentFrame != null) &&
                            !interpreter.IsGlobalCallFrame(currentFrame))
                        {
                            for (int argumentIndex = 1; argumentIndex < arguments.Count; argumentIndex++)
                            {
                                string name = arguments[argumentIndex];

                                code = RuntimeOps.LinkVariable(interpreter, currentFrame,
                                    name, interpreter.GlobalFrame, name, ref result);

                                if (code != ReturnCode.Ok)
                                    break;
                            }
                        }
                        else
                        {
                            // already in global scope... this is a NOP.
                            result = String.Empty;
                        }
                    }
                    else
                    {
                        result = "wrong # args: should be \"global varName ?varName ...?\"";
                        code = ReturnCode.Error;
                    }
                }
                else
                {
                    result = "invalid argument list";
                    code = ReturnCode.Error;
                }
            }
            else
            {
                result = "invalid interpreter";
                code = ReturnCode.Error;
            }

            return code;
        }
        #endregion
    }
}
