/*
 * EncodingDictionary.cs --
 *
 * Copyright (c) 2007-2012 by Joe Mistachkin.  All rights reserved.
 *
 * See the file "license.terms" for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: $
 */

#if SERIALIZATION
using System;
#endif

using System.Collections.Generic;

#if SERIALIZATION
using System.Runtime.Serialization;
#endif

using System.Text;
using Eagle._Attributes;
using Eagle._Components.Private;
using Eagle._Components.Public;
using Eagle._Constants;
using Eagle._Containers.Public;

namespace Eagle._Containers.Private
{
#if SERIALIZATION
    [Serializable()]
#endif
    [ObjectId("78930c01-9509-4f65-89d6-3e1f03a6e0fd")]
    internal sealed class EncodingDictionary : Dictionary<string, Encoding>
    {
        public EncodingDictionary()
            : base()
        {
            // do nothing.
        }

        ///////////////////////////////////////////////////////////////////////

        #region Protected Constructors
#if SERIALIZATION
        private EncodingDictionary(
            SerializationInfo info,
            StreamingContext context
            )
            : base(info, context)
        {
            // do nothing.
        }
#endif
        #endregion

        ///////////////////////////////////////////////////////////////////////

        #region Dead Code
#if DEAD_CODE
        #region Private Constructors
        private EncodingDictionary(
            IDictionary<string, Encoding> dictionary
            )
            : base(dictionary)
        {
            // do nothing.
        }
        #endregion
#endif
        #endregion

        ///////////////////////////////////////////////////////////////////////

        public EncodingDictionary(
            IEqualityComparer<string> comparer
            )
            : base(comparer)
        {
            // do nothing.
        }

        ///////////////////////////////////////////////////////////////////////

        public void Add(
            IDictionary<string, Encoding> dictionary,
            bool strict
            )
        {
            foreach (KeyValuePair<string, Encoding> pair in dictionary)
                if (strict || !this.ContainsKey(pair.Key))
                    this.Add(pair.Key, pair.Value);
        }

        ///////////////////////////////////////////////////////////////////////

        public void Add(
            EncodingInfo[] collection,
            bool strict
            )
        {
            foreach (EncodingInfo item in collection)
                if (strict || !this.ContainsKey(item.Name))
                    this.Add(item.Name, item.GetEncoding());
        }

        ///////////////////////////////////////////////////////////////////////

        public string ToString(
            string pattern,
            bool noCase
            )
        {
            StringList list = new StringList(this.Keys);

            return GenericOps<string>.ListToString(
                list, Index.Invalid, Index.Invalid, ToStringFlags.None,
                Characters.Space.ToString(), pattern, noCase);
        }

        ///////////////////////////////////////////////////////////////////////

        #region System.Object Overrides
        public override string ToString()
        {
            return ToString(null, false);
        }
        #endregion
    }
}
